#include <cstdlib>
#include <msp/strings/utils.h>
#include <msp/time/utils.h>
#include "dummy.h"

using namespace std;
using namespace Msp;

namespace R2C2 {

Dummy::Dummy(const Options &opts):
	power(true),
	turnout_delay(opts.get("turnout_delay", 0.0f)*Time::sec),
	turnout_fail_rate(opts.get("turnout_fail_rate", 0.0f)*RAND_MAX)
{ }

void Dummy::set_power(bool p)
{
	power = p;
	signal_power.emit(power);
}

void Dummy::halt(bool h)
{
	halted = h;
	if(halted)
	{
		for(map<unsigned, LocoState>::iterator i=locos.begin(); i!=locos.end(); ++i)
			if(i->second.speed)
				set_loco_speed(i->first, 0);
	}
	signal_halt.emit(halted);
}

const char *Dummy::enumerate_protocols(unsigned i) const
{
	if(i==0)
		return "dummy";
	return 0;
}

unsigned Dummy::get_protocol_speed_steps(const string &) const
{
	return 0;
}

unsigned Dummy::add_turnout(unsigned addr, const TrackType &)
{
	turnouts[addr];
	return addr;
}

void Dummy::set_turnout(unsigned addr, unsigned state)
{
	TurnoutState &turnout = turnouts[addr];
	if(turnout.state==state && turnout.pending==state)
	{
		signal_turnout.emit(addr, state);
		return;
	}

	turnout.pending = state;
	if(turnout_delay)
		turnout.timeout = Time::now()+turnout_delay;
	else
	{
		if(!turnout_fail_rate || rand()>=turnout_fail_rate)
			turnout.state = state;
		signal_turnout.emit(addr, state);
	}
}

unsigned Dummy::get_turnout(unsigned addr) const
{
	map<unsigned, TurnoutState>::const_iterator i = turnouts.find(addr);
	if(i!=turnouts.end())
		return i->second.state;
	return 0;
}

void Dummy::set_loco_speed(unsigned addr, unsigned speed)
{
	if(speed && halted)
		return;

	LocoState &loco = locos[addr];
	loco.speed = speed;
	signal_loco_speed.emit(addr, speed, loco.reverse);
}

void Dummy::set_loco_reverse(unsigned addr, bool rev)
{
	LocoState &loco = locos[addr];
	loco.reverse = rev;
	signal_loco_speed.emit(addr, loco.speed, rev);
}

void Dummy::set_loco_function(unsigned addr, unsigned func, bool state)
{
	signal_loco_function.emit(addr, func, state);
}

void Dummy::set_sensor(unsigned addr, bool state)
{
	if(sensors[addr]!=state)
	{
		sensors[addr] = state;
		signal_sensor.emit(addr, state);
	}
}

bool Dummy::get_sensor(unsigned addr) const
{
	map<unsigned, bool>::const_iterator i = sensors.find(addr);
	if(i!=sensors.end())
		return i->second;
	return false;
}

void Dummy::tick()
{
	Time::TimeStamp t = Time::now();
	for(map<unsigned, TurnoutState>::iterator i=turnouts.begin(); i!=turnouts.end(); ++i)
	{
		if(i->second.timeout && t>=i->second.timeout)
		{
			if(turnout_fail_rate && rand()<turnout_fail_rate)
			{
				signal_turnout_failed.emit(i->first);
				i->second.pending = i->second.state;
			}
			i->second.state = i->second.pending;
			i->second.timeout = Time::TimeStamp();
			signal_turnout.emit(i->first, i->second.state);
		}
	}
}

} // namespace R2C2
