#include <cmath>
#include "profile.h"

using namespace std;
using namespace Msp;

namespace R2C2 {

void Profile::append_vertex(const Vector &p, bool smooth)
{
	if(vertices.size()>1 && !vertices.back().smooth)
		vertices.push_back(vertices.back());

	Vertex v;
	v.pos = p;
	v.smooth = (!vertices.empty() && smooth);

	if(!vertices.empty())
	{
		Vector span = p-vertices.back().pos;
		span.normalize();
		v.normal.x = span.y;
		v.normal.y = -span.x;

		if(vertices.back().smooth)
		{
			Vector &n = vertices.back().normal;
			n += v.normal;
			n.normalize();
		}
		else
			vertices.back().normal = v.normal;
	}

	vertices.push_back(v);

	if(vertices.size()==1)
	{
		min_coords = p;
		max_coords = p;
	}
	else
	{
		min_coords.x = min(min_coords.x, p.x);
		min_coords.y = min(min_coords.y, p.y);
		max_coords.x = max(max_coords.x, p.x);
		max_coords.y = max(max_coords.y, p.y);
	}
}

const Profile::Vertex &Profile::get_vertex(unsigned i) const
{
	if(i>=vertices.size())
		throw out_of_range("Profile::get_vertex");
	return vertices[i];
}


Profile::Loader::Loader(Profile &p):
	DataFile::ObjectLoader<Profile>(p)
{
	add("point", &Loader::point);
	add("smooth_point", &Loader::smooth_point);
}

void Profile::Loader::point(float x, float y)
{
	obj.append_vertex(Vector(x/1000, y/1000, 0), false);
}

void Profile::Loader::smooth_point(float x, float y)
{
	obj.append_vertex(Vector(x/1000, y/1000, 0), true);
}

} // namespace R2C2
