#ifndef LIBR2C2_PROFILE_H_
#define LIBR2C2_PROFILE_H_

#include <vector>
#include <msp/datafile/objectloader.h>
#include "geometry.h"

namespace R2C2 {

class Profile
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<Profile>
	{
	public:
		Loader(Profile &);
	private:
		void point(float, float);
		void smooth_point(float, float);
	};

	struct Vertex
	{
		Vector pos;
		Vector normal;
		bool smooth;
	};

private:
	std::vector<Vertex> vertices;
	Vector min_coords;
	Vector max_coords;

public:
	void append_vertex(const Vector &, bool);
	unsigned get_n_vertices() const { return vertices.size(); }
	const Vertex &get_vertex(unsigned) const;
	const Vector &get_min_coords() const { return min_coords; }
	const Vector &get_max_coords() const { return max_coords; }
	float get_width() const { return max_coords.x-min_coords.x; }
	float get_height() const { return max_coords.y-min_coords.y; }
};

} // namespace R2C2

#endif
