#ifndef LIBR2C2_SENSOR_H_
#define LIBR2C2_SENSOR_H_

#include <sigc++/signal.h>
#include <msp/time/timedelta.h>

namespace R2C2 {

class Block;
class Layout;

class Sensor: public sigc::trackable
{
public:
	enum State
	{
		INACTIVE,
		MAYBE_INACTIVE,
		MAYBE_ACTIVE,
		ACTIVE
	};

	sigc::signal<void, State> signal_state_changed;

protected:
	Layout &layout;
	unsigned address;
	unsigned id;
	bool invert;
	State state;
	Msp::Time::TimeDelta state_confirm_timeout;

	Sensor(Layout &);
public:
	virtual ~Sensor();

	virtual void set_address(unsigned);
	unsigned get_address() const { return address; }
	State get_state() const { return state; }
	virtual Block *get_block() const { return 0; }

	void tick(const Msp::Time::TimeDelta &);

private:
	void event(unsigned, bool);
};

} // namespace R2C2

#endif
